pub fn max_or_undefined<T>(a: T, b: T) -> Twhere
T: IntrinsicOrd,Expand description
Partial maximum of types with intrinsic representations for undefined.
See the IntrinsicOrd trait.
pub fn max_or_undefined<T>(a: T, b: T) -> Twhere
T: IntrinsicOrd,Partial maximum of types with intrinsic representations for undefined.
See the IntrinsicOrd trait.