pub fn min_or_undefined<T>(a: T, b: T) -> Twhere
T: IntrinsicOrd,Expand description
Partial minimum of types with intrinsic representations for undefined.
See the IntrinsicOrd trait.
pub fn min_or_undefined<T>(a: T, b: T) -> Twhere
T: IntrinsicOrd,Partial minimum of types with intrinsic representations for undefined.
See the IntrinsicOrd trait.